# -*- encoding: utf-8 -*-
#################################################################################
#                                                                               #
#    Model module for OpenERP                                                   #
#    Copyright (C) 2010 Sébastien BEAU <sebastien.beau@akretion.comr>           #
#                                                                               #
#    This program is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU Affero General Public License as             #
#    published by the Free Software Foundation, either version 3 of the         #
#    License, or (at your option) any later version.                            #
#                                                                               #
#    This program is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of             #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              #
#    GNU Affero General Public License for more details.                        #
#                                                                               #
#    You should have received a copy of the GNU Affero General Public License   #
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.      #
#                                                                               #
#################################################################################

from osv import osv, fields
import netsvc
from tools.translate import _
from datetime import datetime
from dateutil.relativedelta import relativedelta

class stock_warehouse_orderpoint(osv.osv):
    _inherit = 'stock.warehouse.orderpoint'

# the function get_qty_to_procure is a native function extracted from the big function _procure_orderpoint_confirm in the file addons/procurement/scheduler.py

    def get_qty_to_procure(self, cr, uid, op, prods, context=None):
        '''
        Return the qty of product to procure

        @param self: The object pointer
        @param cr: The current row, from the database cursor,
        @param op: the orderpoint object take care it's not an id this is used in order to don't make a browse request for each orderpoint
        @param prods: The virtual product quantity in the stock
        @param context: A standard dictionary for contextual values
        @return:  a quantity
        """
        '''

        qty = max(op.product_min_qty, op.product_max_qty)-prods

        reste = qty % op.qty_multiple
        if reste > 0:
            qty += op.qty_multiple - reste

        newdate = datetime.today() + relativedelta(days = int(op.product_id.seller_delay))
        if qty <= 0:
            return False
        if op.product_id.type not in ('consu'):
            if op.procurement_draft_ids:
            # Check draft procurement related to this order point
                pro_ids = [x.id for x in op.procurement_draft_ids]
                cr.execute('select id, product_qty from procurement_order where id in %s order by product_qty desc', (tuple(pro_ids), ))
                procure_datas = cr.dictfetchall()
                to_generate = qty
                for proc_data in procure_datas:
                    if to_generate >= proc_data['product_qty']:
                        wf_service.trg_validate(uid, 'procurement.order', proc_data['id'], 'button_confirm', cr)
                        procurement_obj.write(cr, uid, [proc_data['id']],  {'origin': op.name}, context=context)
                        to_generate -= proc_data['product_qty']
                    if not to_generate:
                        break
                qty = to_generate
        return qty

    def update_procurement(self, cr, uid, op, context):
        if op.procurement_id.purchase_id.state == 'confirmed':
            return True
        location_obj = self.pool.get('stock.location')
        prods = location_obj._product_virtual_get(cr, uid,
                        op.location_id.id, [op.product_id.id],
                        {'uom': op.product_uom.id})[op.product_id.id]

        if prods < op.product_min_qty:        
            qty = self.get_qty_to_procure(cr, uid, op, prods, context)

            if qty:
                total_qty = op.procurement_id.product_qty + qty
                self.pool.get('procurement.order').write(cr, uid, op.procurement_id.id, {'product_qty' : total_qty})
                self.pool.get('stock.move').write(cr, uid, op.procurement_id.move_id.id, {'product_qty' : total_qty})
                self.pool.get('purchase.order.line').write(cr, uid, op.procurement_id.purchase_id.order_line[0].id, {'product_qty' : total_qty})

        return True

stock_warehouse_orderpoint()
